// https://www.twilio.com/blog/how-to-build-a-cli-with-node-js

import { fontawesomeSubset } from 'fontawesome-subset';
import chokidar from 'chokidar'
import fastGlob from 'fast-glob'
import fs from 'fs'
import arg from 'arg'
import path from 'path'

function parseArgumentsIntoOptions(rawArgs) {
  const args = arg(
    {
      '--watch': Boolean,
      '-w': '--watch'
    },
    {
      argv: rawArgs.slice(2),
    }
  );
  return {
    watch: args['--watch'] || false,
  };
 }

export function cli(args) {
  let options = parseArgumentsIntoOptions(args)
  
  console.log(options)

  copyIcons()

  if (options.watch) {
    process.stdin.on('end', () => process.exit(0))

    process.stdin.resume()
  
    startWatcher()
  }
}

async function copyIcons() {
  let start = process.hrtime()

  console.log('Copying icons...')

  let icons = { }
  
  const files = fastGlob.sync(['app/views/**/*'])

  for (let file of files) {
    let content = fs.readFileSync(path.resolve(file), 'utf8')

    let results = [...content.matchAll(/icon\(:(?<type>fa[sbltrd]), :(?<name>[a-z|_]*)[,)]/gm)]

    for (let result of results) {

      let icon_type = null
      
      switch(result.groups.type) {
        case 'fas':
          icon_type = 'solid'
          break
        case 'far':
          icon_type = 'regular'
          break
        case 'fab':
          icon_type = 'brands'
          break
        case 'fal':
          icon_type = 'light'
          break
        case 'fad':
          icon_type = 'duotone'
          break
      }
      let icon_name = result.groups.name

      if (icons[icon_type] === undefined)
        icons[icon_type] = []

      if(!icons[icon_type].includes(icon_name))
        icons[icon_type].push(icon_name)
    }
  }

  fontawesomeSubset(icons, 'app/assets/fonts', { package: 'pro' })

  console.log('Done in ' + (process.hrtime(start)[1] / 1e6).toFixed(3) + 'ms.');
}

async function startWatcher() {
  
  let watcher = chokidar.watch(['app/views/**/*.slim'], { ignoreInitial: true })

  watcher.on('change', async (file) => {
    copyIcons()
  })

  watcher.on('add', async (file) => {
    copyIcons()
  })

}